#include <windows.h>
#include <tchar.h>


/* Return a malloced wide char string from an UTF-8 encoded input
   string STRING.  Caller must free this value.  Returns NULL on
   failure.  Caller may use GetLastError to get the actual error
   number.  The result of calling this function with STRING set to
   NULL is not defined.  */
static wchar_t *
utf8_to_wchar (const char *string)
{
  int n;
  size_t length = strlen (string);
  wchar_t *result;
  size_t nbytes;

  n = MultiByteToWideChar (CP_UTF8, 0, string, length, NULL, 0);
  if (n < 0 || (n+1) <= 0)
    {
      SetLastError (ERROR_INVALID_PARAMETER);
      return NULL;
    }

  nbytes = (size_t)(n+1) * sizeof(*result);
  if (nbytes / sizeof(*result) != (n+1)) 
    {
      SetLastError (ERROR_INVALID_PARAMETER);
      return NULL;
    }
  result = malloc (nbytes);
  n = MultiByteToWideChar (CP_UTF8, 0, string, length, result, n);
  if (n < 0)
    return NULL;
  result[n] = 0;
  
  return result;
}


static BOOL
my_DeleteFileA (LPCSTR lpFileName)
{
  wchar_t *filename;
  BOOL result;
  int err;

  filename = utf8_to_wchar (lpFileName);
  if (!filename)
    return FALSE;

  result = DeleteFileW (filename);

  err = GetLastError ();
  free (filename);
  SetLastError (err);
  return result;
}


int
_wremove(const wchar_t *path)
{
  if (DeleteFileW(path))
    return 0;
  return -1;
}


int
remove(const char *path)
{
  if (my_DeleteFileA(path))
    return 0;
  return -1;
}

void perror(const char* str)
{
  LPVOID buf;
  if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER
                    | FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,
                    GetLastError(),
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                    (LPTSTR) &buf,
                    0,
                    NULL)) {
    _ftprintf(stderr, _T("%s: %s"), str, buf);
    fflush(stderr);
    LocalFree(buf);
  }
  else
    _ftprintf(stderr, _T("%s: unknown Windows error\n"), str);
}
